<?php
/**
 * @file
 * Installation profile for Volunteer Rally.
 */
if (!function_exists('profiler_v2')) {
  require 'libraries/profiler/profiler.inc';
}
profiler_v2('volunteer_rally');

/**
 * Since profiler installs modules in alphabetical order, jquery_ui
 * does not find the libraries module. This gets included here so
 * requirements pass.
 */
require_once 'modules/contrib/libraries/libraries.module';

/**
 * Return a description of the profile for the initial installation screen.
 *
 * @return
 *   An array with keys 'name' and 'description' describing this profile,
 *   and optional 'language' to override the language selection for
 *   language-specific profiles.
 */
function volunteer_rally_profile_details() {
  return array(
    'name' => 'Volunteer Rally',
    'description' => st('Select this profile to enable a Volunteer Rally installation.'),
  );
}

/**
 * Implementation of hook_install().
 */
function volunteer_rally_install() {
  $types = array(
    array(
      'type' => 'page',
      'name' => st('Page'),
      'module' => 'node',
      'description' => st("A <em>page</em> is a simple method for creating and displaying information that rarely changes, such as an \"About us\" section of a website. By default, a <em>page</em> entry does not allow visitor comments and is not featured on the site's initial home page."),
      'custom' => TRUE,
      'modified' => TRUE,
      'locked' => FALSE,
      'help' => '',
      'min_word_count' => '',
    ),
  );

  foreach ($types as $type) {
    $type = (object) _node_type_set_defaults($type);
    node_type_save($type);
  }

  // Default page to not be promoted, and have comments disabled, and create new revisions.
  variable_set('node_options_page', array('status', 'revision'));
  variable_set('comment_page', COMMENT_NODE_DISABLED);

  // Don't display date and author information for anything by default.
  $theme_settings = variable_get('theme_settings', array());
  $types = array(
    'group',
    'page',
    'priority_code',
    'profile',
    'shift',
    'signup',
  );
  foreach ($types as $type) {
    $theme_settings['toggle_node_info_' . $type] = FALSE;
  }
  $theme_settings['toggle_search'] = 0;
  variable_set('theme_settings', $theme_settings);

  // Create roles.
  _volunteer_rally_user_roles();
  // Assign sensible input filter defaults to roles.
  _volunteer_rally_better_formats();
  // Initial permissions.
  _volunteer_rally_set_permissions();
  // Pathauto defaults.
  _volunteer_rally_pathauto();
  // Core configuration and tweaks.
  _volunteer_rally_core();

  // Some basic settings for Iggy.
  $settings = array(
    'toggle_search' => 0,
    'default_logo' => 1,
  );
  variable_set('theme_iggy_settings', $settings);

  // Design kit
  $settings = array(
    'primary' => '#38393a',
    'background' => '#bdbdbd',
  );
  variable_set('designkit_color', $settings);
}

/**
 * Implementation of hook_form_alter().
 *
 * Allows the profile to alter the site-configuration form. This is
 * called through custom invocation, so $form_state is not populated.
 */
function volunteer_rally_form_alter(&$form, $form_state, $form_id) {
  if ($form_id == 'install_configure') {
    // Set default for site name field.
    $form['site_information']['site_name']['#default_value'] = $_SERVER['SERVER_NAME'];
  }
}

/**
 * Creates Volunteer Coordinator and Administrator roles.
 */
function _volunteer_rally_user_roles() {
  foreach (array('volunteer coordinator', 'administrator') as $role) {
    if (!db_result(db_query("SELECT rid FROM {role} WHERE name = '%s'", array(':role_name' => $role)))) {
      db_query("INSERT INTO {role} (name) VALUES ('%s')", $role);
      drupal_set_message(t('The %role role has been added.', array('%role' => $role)));
    }
  }
}

/**
 * Set volunteer coordinator and administrator default input format to full HTML.
 */
function _volunteer_rally_better_formats() {
  $roles = array();
  foreach (user_roles() as $rid => $name) {
    if (in_array($name, array('volunteer coordinator', 'administrator'))) {
      $roles[] = $rid;
      // Float admin to top, volunteer coordinator 2nd highest.
      $weight = -2 * $rid;
      db_query("UPDATE {better_formats_defaults} SET format = %d, weight = %d WHERE rid = %d AND type = '%s'", array(':format' => 2, ':weight' => $weight, ':rid' => $rid, ':type' => 'node'));
      db_query("UPDATE {better_formats_defaults} SET format = %d, weight = %d WHERE rid = %d AND type = '%s'", array(':format' => 2, ':weight' => $weight, ':rid' => $rid, ':type' => 'comment'));
    }
  }
  $current = db_result(db_query("SELECT roles FROM {filter_formats} WHERE format = 2"));
  if ($current) {
    $current = explode(',', $roles);
    $roles = array_merge($current, $roles);
  }
  $roles = ','. implode(',', $roles) .',';
  // Allow volunteer coordinators and administrators to use HTML;
  db_query("UPDATE {filter_formats} SET roles = '%s' WHERE format = 2", array(':roles' => $roles));
}

/**
 * Set some sensible permissions.
 */
function _volunteer_rally_set_permissions() {
  $roles = user_roles();
  $admin_rid = array_search('administrator', $roles);
  $admin_user_perms = array(
    'access administration menu',
    'administer blocks',
    'create url aliases',
    'administer menu',
    'administer nodes',
    'create page content',
    'delete any page content',
    'edit any page content',
    'revert revisions',
    'view revisions',
    'assign volunteer coordinator role',
    'assign administrator role',
    'administer users',
    'access administration pages',
    'access user profiles',
  );
  if (!db_result(db_query("SELECT rid FROM {permission} LEFT JOIN {role} USING (rid) WHERE name = '%s'", array(':role_name' => 'administrator')))) {
    db_query("INSERT INTO {permission} (rid, perm) VALUES (%d, '%s')", array(':rid' => $admin_rid, implode(', ', $admin_user_perms)));
    drupal_set_message(t("Set sensible defaults for %role role.", array('%role' => 'administrator')));
  }

  $volunteer_coordinator_rid = array_search('volunteer coordinator', $roles);
  $volunteer_coordinator_user_perms = array(
    'access administration pages',
    'access user profiles',
    'administer users',
    'administer nodes', // This is needed so that volunteer coordinators can change the 'author' of signup nodes
    'create page content',
    'delete own page content',
    'edit any page content',
    'revert revisions',
    'view revisions',
  );
  if (!db_result(db_query("SELECT rid FROM {permission} LEFT JOIN {role} USING (rid) WHERE name = '%s'", array(':role_name' => 'volunteer coordinator')))) {
    db_query("INSERT INTO {permission} (rid, perm) VALUES (%d, '%s')", array(':rid' => $volunteer_coordinator_rid, implode(', ', $volunteer_coordinator_user_perms)));
    drupal_set_message(t("Set sensible defaults for %role role.", array('%role' => 'volunteer coordinator')));
  }
}

/**
 * Initial settings for pathauto and path redirect.
 */
function _volunteer_rally_pathauto() {
  // Get rid of wonky content/foo pattern.
  // The rationale is that it's easier to bulk-update pathauto aliases
  // than it is to remove unwanted ones.
  // We leave this blank because there's no one-size-fits all pattern.
  variable_set('pathauto_node_pattern', '');
  // Remove story pattern.  Even though we didn't create a story type,
  // pathauto will insert this default anyway.
  variable_del('pathauto_node_story_pattern');

  // Path redirect settings.
  $conf['path_redirect_allow_bypass'] = 0;
  $conf['path_redirect_auto_redirect'] = 1;
  $conf['path_redirect_default_status'] = '301';
  $conf['path_redirect_purge_inactive'] = '31536000'; // 1 year.
  $conf['path_redirect_redirect_warning'] = 0;
  foreach ($conf as $var => $val) {
    variable_set($var, $val);
  }
}

/**
 * Core customization.
 */
function _volunteer_rally_core() {
  // Change default anonymous to "Visitor".
  variable_set('anonymous', 'Visitor');
  
  // Increase the capacity of the Drupal watchdog.
  // The default of 1000 rows overflows too quickly, sometimes losing important
  // debug information.  100k rows is big, yet should still keep the watchdog
  // table under 10MB.
  variable_set('dblog_row_limit', 100000);
  
  // Set to open registration.
  variable_set('user_register', 1);

  // Disable all blocks.
  db_query("UPDATE {blocks} SET status = 0, region = ''");

  // Date formats are set here or the homepage calendar view breaks.
  variable_set('date_format_short', 'M j Y - g:ia');
  variable_set('date_format_long', 'l, F j, Y - H:i');
  variable_set('date_format_medium', 'D, Y-m-d H:i');
  variable_set('date_format_day', 'j F Y');
  variable_set('date_format_time', 'g:ia');

  // Timezones.
  if (variable_get('date_default_timezone', FALSE) === FALSE) {
    variable_set('date_default_timezone', date('Z'));
  }
  // Set for the date module as well.
  $timezone = date('e');
  variable_set('date_default_timezone_name', date('e'));

  // Disable per-user timezones for simplicity.
  variable_set('configurable_timezones', 0);

  // Set first day to 0 (date and calendar modules have seemingly
  // random defaults http://drupal.org/node/552154).
  variable_set('date_first_day', 0);

  // Add sign-in/logout link to primary.
  _volunteer_rally_menu_items();

  // Placeholder About page.
  _volunteer_rally_create_nodes();

  // Clean-up tasks.
  _volunteer_rally_clean();
}

/**
 * Clear and rebuild caches.
 */
function _volunteer_rally_clean() {
  // Since content_profile adds a value for this variable during
  // install, we must delete it here.
  variable_del('content_profile_profile');

  // Set default theme. This needes some more set up on next page load
  // We cannot do everything here because of _system_theme_data() static cache
  system_theme_data();
  db_query("UPDATE {system} SET status = 0 WHERE type = 'theme' AND name ='%s'", 'garland');
  db_query("UPDATE {system} SET status = 1 WHERE type = 'theme' AND name ='%s'", 'iggy');
  db_query("UPDATE {blocks} SET status = 0, region = ''"); // disable all DB blocks

  // Rebuild key tables/caches
  module_rebuild_cache(); // Detects the newly added bootstrap modules
  node_access_rebuild();
  drupal_get_schema(NULL, TRUE); // Clear schema DB cache
  drupal_flush_all_caches();    
  system_theme_data();  // Rebuild theme cache.
  _block_rehash();      // Rebuild block cache.
  views_invalidate_cache(); // Rebuild the views.
  menu_rebuild();       // Rebuild the menu.
  features_rebuild();   // Features rebuild scripts.
  node_access_needs_rebuild(FALSE);
}

/**
 * Menu items.
 */
function _volunteer_rally_menu_items() {
  $primary = array(
    'user/login' => t('Sign In'),
    'logout' => t('Log out'),
  );
  $secondary = array(
    'search' => t('Search'),
    'admin/user/user/create' => t('Create new user'),
  );
  $weight = 42;
  foreach ($primary as $path => $title) {
    $link = array(
      'menu_name'     => 'primary-links',
      'weight'        => $weight,
      'link_title'    => $title,
      'options'       => array(),
      'module'        => 'menu',
      'link_path'     => drupal_get_normal_path($path),
      'hidden'        => 0,
      'has_children'  => 0,
      'expanded'      => 0,
      'customized'    => 0,
      'updated'       => 0,
    );

    // Since menu_save_link will do a query if any plid is passed.
    // Omit it if this is a top level link.
    if ($plid) {
      $link['plid'] = $plid;
    }

    $return = menu_link_save($link);
    $weight ++;
  }

  $weight = -5;
  foreach ($secondary as $path => $title) {
    $link = array(
      'menu_name'     => 'secondary-links',
      'weight'        => $weight,
      'link_title'    => $title,
      'options'       => array(),
      'module'        => 'menu',
      'link_path'     => drupal_get_normal_path($path),
      'hidden'        => 0,
      'has_children'  => 0,
      'expanded'      => 0,
      'customized'    => 0,
      'updated'       => 0,
    );

    // Since menu_save_link will do a query if any plid is passed.
    // Omit it if this is a top level link.
    if ($plid) {
      $link['plid'] = $plid;
    }

    $return = menu_link_save($link);
    $weight += 10;
  }
}

/**
 * Create placeholder About node.
 */
function _volunteer_rally_create_nodes() {
  $node = new stdClass;
  $node->uid = 1;
  $node->status = 1;
  $node->type = 'page';
  $node->title = st('About');

  // Node body stored separately.
  include_once dirname(__FILE__) . '/includes/about.inc';
  $node->body = _volunteer_rally_about_node();
  $node->format = 2;

  // Need to fake uid 1 in order to add a node path.
  global $user;
  $user->uid = 1;
  $node->path = 'about';

  // Menu entry.
  $node->menu['link_title'] = st('About');
  $node->menu['menu_name'] = 'primary-links';
  $node->menu['weight'] = 42;

  node_save($node);
}

/**
 * Set Volunteer Rally as the default install profile.
 */
if (!function_exists('system_form_install_select_profile_form_alter')) {
  function system_form_install_select_profile_form_alter(&$form, $form_state) {
    foreach($form['profile'] as $key => $element) {
      $form['profile'][$key]['#value'] = 'volunteer_rally';
    }
  }
}
